/* Generated by Cython 0.25.2 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03020000)
    #error Cython requires Python 2.6+ or Python 3.2+.
#else
#define CYTHON_ABI "0_25_2"
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x03030000 || (PY_MAJOR_VERSION == 2 && PY_VERSION_HEX >= 0x02070000)
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_FASTCALL
  #define METH_FASTCALL 0x80
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject **args,
                                              Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE___gldraw_es2
#define __PYX_HAVE_API___gldraw_es2
#include <stddef.h>
#include <stdint.h>
#include "GLES2/gl2.h"
#include <math.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER) && defined (_M_X64)
    #define __Pyx_sst_abs(value) _abs64(value)
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyObject_AsSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#if PY_MAJOR_VERSION < 3
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u)
{
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#else
#define __Pyx_Py_UNICODE_strlen Py_UNICODE_strlen
#endif
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */

static PyObject *__pyx_m;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "build/temp.linux-x86_64-3.5/pybiklib/ext/_gldraw_es2.pyx",
};

/*--- Type declarations ---*/
struct __pyx_t_11_gldraw_es2_Block;
struct __pyx_t_11_gldraw_es2_Cube;

/* "_gldraw_es2.pxd":4
 * 
 * from gl_es2 cimport *    #line 34
 * cdef enum: #    #line 67             # <<<<<<<<<<<<<<
 *     MAX_TRANSFORMATIONS = 60    #line 71
 *     MAX_BLOCKS = 1312    #line 72
 */
enum  {
  __pyx_e_11_gldraw_es2_MAX_TRANSFORMATIONS = 60,
  __pyx_e_11_gldraw_es2_MAX_BLOCKS = 0x520,
  __pyx_e_11_gldraw_es2_MAX_FACES = 20,
  __pyx_e_11_gldraw_es2_ATTRIB_LOCATION = 0,
  __pyx_e_11_gldraw_es2_PICKATTRIB_LOCATION = 5,
  __pyx_e_11_gldraw_es2_ATTRIB_CNT = 7
};

/* "_gldraw_es2.pxd":11
 *     PICKATTRIB_LOCATION = 5    #line 75
 *     ATTRIB_CNT = 7    #line 76
 * ctypedef float vec4[4]    #line 79             # <<<<<<<<<<<<<<
 * ctypedef vec4 mat4[4]    #line 80
 * cdef void matrix_set_identity(mat4 &matrix) nogil    #line 87
 */
typedef float __pyx_t_11_gldraw_es2_vec4[4];

/* "_gldraw_es2.pxd":12
 *     ATTRIB_CNT = 7    #line 76
 * ctypedef float vec4[4]    #line 79
 * ctypedef vec4 mat4[4]    #line 80             # <<<<<<<<<<<<<<
 * cdef void matrix_set_identity(mat4 &matrix) nogil    #line 87
 * cdef void init_gldraw()    #line 137
 */
typedef __pyx_t_11_gldraw_es2_vec4 __pyx_t_11_gldraw_es2_mat4[4];

/* "_gldraw_es2.pyx":54
 * 
 * 
 * cdef enum: # #px/             # <<<<<<<<<<<<<<
 * #if True:
 *     DEBUG_MSGEXT = 8
 */
enum  {
  __pyx_e_11_gldraw_es2_DEBUG_MSGEXT = 8
};

/* "_gldraw_es2.pyx":93
 *     matrix[3][0] = 0.; matrix[3][1] = 0.; matrix[3][2] = 0.; matrix[3][3] = 1.
 * 
 * cdef struct Block: #px/             # <<<<<<<<<<<<<<
 * #class Block:     # pylint: disable=R0903
 * ##px-
 */
struct __pyx_t_11_gldraw_es2_Block {
  __pyx_t_11_gldraw_es2_vec4 *transformation;
  int in_motion;
  int idx_triangles;
  int cnt_triangles;
};

/* "_gldraw_es2.pyx":108
 *         #self.cnt_triangles = None
 * 
 * cdef struct Cube: #px/             # <<<<<<<<<<<<<<
 * #class cube:     # pylint: disable=W0232, R0903
 *     mat4 transformations[MAX_TRANSFORMATIONS] #px/
 */
struct __pyx_t_11_gldraw_es2_Cube {
  __pyx_t_11_gldraw_es2_mat4 transformations[__pyx_e_11_gldraw_es2_MAX_TRANSFORMATIONS];
  unsigned int number_blocks;
  struct __pyx_t_11_gldraw_es2_Block blocks[__pyx_e_11_gldraw_es2_MAX_BLOCKS];
  int cnt_pick;
  int idx_debug;
  int cnt_debug;
  GLuint object_location;
  GLuint glbuffer;
  int vertexdatalen;
  char *vertexdata;
  long attrib_pointers[__pyx_e_11_gldraw_es2_ATTRIB_CNT];
  __pyx_t_11_gldraw_es2_mat4 rotation_matrix;
};

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* GetModuleGlobalName.proto */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = PyThreadState_GET();
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_int(unsigned int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntFromPy.proto */
static CYTHON_INLINE unsigned int __Pyx_PyInt_As_unsigned_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* FunctionExport.proto */
static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'libc.stddef' */

/* Module declarations from 'libc.stdint' */

/* Module declarations from 'gl_es2' */

/* Module declarations from 'libc.math' */

/* Module declarations from '_gldraw_es2' */
static long __pyx_v_11_gldraw_es2_debug;
static struct __pyx_t_11_gldraw_es2_Cube __pyx_v_11_gldraw_es2_cube;
static void __pyx_f_11_gldraw_es2_matrix_set_identity(__pyx_t_11_gldraw_es2_vec4 *); /*proto*/
static void __pyx_f_11_gldraw_es2_matrix_mult(__pyx_t_11_gldraw_es2_vec4 *, __pyx_t_11_gldraw_es2_vec4 *, __pyx_t_11_gldraw_es2_vec4 *); /*proto*/
static void __pyx_f_11_gldraw_es2__gl_enable_pointer(GLuint, GLint, GLenum, GLboolean, long); /*proto*/
static void __pyx_f_11_gldraw_es2__gl_disable_pointer(GLuint); /*proto*/
#define __Pyx_MODULE_NAME "_gldraw_es2"
int __pyx_module_is_main__gldraw_es2 = 0;

/* Implementation of '_gldraw_es2' */
static PyObject *__pyx_builtin_print;
static PyObject *__pyx_builtin_range;
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_print[] = "print";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_module[] = "module";
static const char __pyx_k_package[] = "__package__";
static const char __pyx_k_compiled[] = "  compiled:";
static const char __pyx_k_compiled_2[] = "__compiled";
static const char __pyx_k_gldraw_es2[] = "_gldraw_es2";
static const char __pyx_k_GL_type_es2[] = "  GL-type: es2";
static const char __pyx_k_init_module[] = "init module:";
static const char __pyx_k_DEBUG_MSGEXT[] = "DEBUG_MSGEXT";
static const char __pyx_k_from_package[] = "  from package:";
static const char __pyx_k_set_debug_flags[] = "set_debug_flags";
static const char __pyx_k_tmp[] = "/tmp/build/temp.linux-x86_64-3.5/pybiklib/ext/_gldraw_es2.pyx";
static PyObject *__pyx_n_s_DEBUG_MSGEXT;
static PyObject *__pyx_kp_u_GL_type_es2;
static PyObject *__pyx_kp_u_compiled;
static PyObject *__pyx_n_s_compiled_2;
static PyObject *__pyx_kp_u_from_package;
static PyObject *__pyx_n_s_gldraw_es2;
static PyObject *__pyx_kp_s_tmp;
static PyObject *__pyx_kp_u_init_module;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_module;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_package;
static PyObject *__pyx_n_s_print;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_set_debug_flags;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_pf_11_gldraw_es2_set_debug_flags(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_module); /* proto */
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_codeobj__3;

/* "_gldraw_es2.pyx":60
 * debug = 0
 * 
 * def set_debug_flags(module):             # <<<<<<<<<<<<<<
 *     global debug
 *     if module.DEBUG_MSGEXT: debug |= DEBUG_MSGEXT
 */

/* Python wrapper */
static PyObject *__pyx_pw_11_gldraw_es2_1set_debug_flags(PyObject *__pyx_self, PyObject *__pyx_v_module); /*proto*/
static PyMethodDef __pyx_mdef_11_gldraw_es2_1set_debug_flags = {"set_debug_flags", (PyCFunction)__pyx_pw_11_gldraw_es2_1set_debug_flags, METH_O, 0};
static PyObject *__pyx_pw_11_gldraw_es2_1set_debug_flags(PyObject *__pyx_self, PyObject *__pyx_v_module) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_debug_flags (wrapper)", 0);
  __pyx_r = __pyx_pf_11_gldraw_es2_set_debug_flags(__pyx_self, ((PyObject *)__pyx_v_module));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11_gldraw_es2_set_debug_flags(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_module) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("set_debug_flags", 0);

  /* "_gldraw_es2.pyx":62
 * def set_debug_flags(module):
 *     global debug
 *     if module.DEBUG_MSGEXT: debug |= DEBUG_MSGEXT             # <<<<<<<<<<<<<<
 * 
 * ##px-
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_module, __pyx_n_s_DEBUG_MSGEXT); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 62, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 62, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {
    __pyx_v_11_gldraw_es2_debug = (__pyx_v_11_gldraw_es2_debug | __pyx_e_11_gldraw_es2_DEBUG_MSGEXT);
  }

  /* "_gldraw_es2.pyx":60
 * debug = 0
 * 
 * def set_debug_flags(module):             # <<<<<<<<<<<<<<
 *     global debug
 *     if module.DEBUG_MSGEXT: debug |= DEBUG_MSGEXT
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("_gldraw_es2.set_debug_flags", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "_gldraw_es2.pyx":87
 * 
 * ##pxm-FUNC PD nogil
 * cdef void matrix_set_identity(mat4 &matrix) nogil:             # <<<<<<<<<<<<<<
 *     matrix[0][0] = 1.; matrix[0][1] = 0.; matrix[0][2] = 0.; matrix[0][3] = 0.
 *     matrix[1][0] = 0.; matrix[1][1] = 1.; matrix[1][2] = 0.; matrix[1][3] = 0.
 */

static void __pyx_f_11_gldraw_es2_matrix_set_identity(__pyx_t_11_gldraw_es2_vec4 *__pyx_v_matrix) {

  /* "_gldraw_es2.pyx":88
 * ##pxm-FUNC PD nogil
 * cdef void matrix_set_identity(mat4 &matrix) nogil:
 *     matrix[0][0] = 1.; matrix[0][1] = 0.; matrix[0][2] = 0.; matrix[0][3] = 0.             # <<<<<<<<<<<<<<
 *     matrix[1][0] = 0.; matrix[1][1] = 1.; matrix[1][2] = 0.; matrix[1][3] = 0.
 *     matrix[2][0] = 0.; matrix[2][1] = 0.; matrix[2][2] = 1.; matrix[2][3] = 0.
 */
  ((__pyx_v_matrix[0])[0]) = 1.;
  ((__pyx_v_matrix[0])[1]) = 0.;
  ((__pyx_v_matrix[0])[2]) = 0.;
  ((__pyx_v_matrix[0])[3]) = 0.;

  /* "_gldraw_es2.pyx":89
 * cdef void matrix_set_identity(mat4 &matrix) nogil:
 *     matrix[0][0] = 1.; matrix[0][1] = 0.; matrix[0][2] = 0.; matrix[0][3] = 0.
 *     matrix[1][0] = 0.; matrix[1][1] = 1.; matrix[1][2] = 0.; matrix[1][3] = 0.             # <<<<<<<<<<<<<<
 *     matrix[2][0] = 0.; matrix[2][1] = 0.; matrix[2][2] = 1.; matrix[2][3] = 0.
 *     matrix[3][0] = 0.; matrix[3][1] = 0.; matrix[3][2] = 0.; matrix[3][3] = 1.
 */
  ((__pyx_v_matrix[1])[0]) = 0.;
  ((__pyx_v_matrix[1])[1]) = 1.;
  ((__pyx_v_matrix[1])[2]) = 0.;
  ((__pyx_v_matrix[1])[3]) = 0.;

  /* "_gldraw_es2.pyx":90
 *     matrix[0][0] = 1.; matrix[0][1] = 0.; matrix[0][2] = 0.; matrix[0][3] = 0.
 *     matrix[1][0] = 0.; matrix[1][1] = 1.; matrix[1][2] = 0.; matrix[1][3] = 0.
 *     matrix[2][0] = 0.; matrix[2][1] = 0.; matrix[2][2] = 1.; matrix[2][3] = 0.             # <<<<<<<<<<<<<<
 *     matrix[3][0] = 0.; matrix[3][1] = 0.; matrix[3][2] = 0.; matrix[3][3] = 1.
 * 
 */
  ((__pyx_v_matrix[2])[0]) = 0.;
  ((__pyx_v_matrix[2])[1]) = 0.;
  ((__pyx_v_matrix[2])[2]) = 1.;
  ((__pyx_v_matrix[2])[3]) = 0.;

  /* "_gldraw_es2.pyx":91
 *     matrix[1][0] = 0.; matrix[1][1] = 1.; matrix[1][2] = 0.; matrix[1][3] = 0.
 *     matrix[2][0] = 0.; matrix[2][1] = 0.; matrix[2][2] = 1.; matrix[2][3] = 0.
 *     matrix[3][0] = 0.; matrix[3][1] = 0.; matrix[3][2] = 0.; matrix[3][3] = 1.             # <<<<<<<<<<<<<<
 * 
 * cdef struct Block: #px/
 */
  ((__pyx_v_matrix[3])[0]) = 0.;
  ((__pyx_v_matrix[3])[1]) = 0.;
  ((__pyx_v_matrix[3])[2]) = 0.;
  ((__pyx_v_matrix[3])[3]) = 1.;

  /* "_gldraw_es2.pyx":87
 * 
 * ##pxm-FUNC PD nogil
 * cdef void matrix_set_identity(mat4 &matrix) nogil:             # <<<<<<<<<<<<<<
 *     matrix[0][0] = 1.; matrix[0][1] = 0.; matrix[0][2] = 0.; matrix[0][3] = 0.
 *     matrix[1][0] = 0.; matrix[1][1] = 1.; matrix[1][2] = 0.; matrix[1][3] = 0.
 */

  /* function exit code */
}

/* "_gldraw_es2.pyx":137
 * 
 * ##pxm-FUNC PD
 * cdef void init_gldraw():             # <<<<<<<<<<<<<<
 *     if debug & DEBUG_MSGEXT:
 *         print('init module:', __name__)
 */

static void __pyx_f_11_gldraw_es2_init_gldraw(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("init_gldraw", 0);

  /* "_gldraw_es2.pyx":138
 * ##pxm-FUNC PD
 * cdef void init_gldraw():
 *     if debug & DEBUG_MSGEXT:             # <<<<<<<<<<<<<<
 *         print('init module:', __name__)
 *         print('  from package:', __package__)
 */
  __pyx_t_1 = ((__pyx_v_11_gldraw_es2_debug & __pyx_e_11_gldraw_es2_DEBUG_MSGEXT) != 0);
  if (__pyx_t_1) {

    /* "_gldraw_es2.pyx":139
 * cdef void init_gldraw():
 *     if debug & DEBUG_MSGEXT:
 *         print('init module:', __name__)             # <<<<<<<<<<<<<<
 *         print('  from package:', __package__)
 *         print('  compiled:', __compiled)
 */
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 139, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 139, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_kp_u_init_module);
    __Pyx_GIVEREF(__pyx_kp_u_init_module);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_kp_u_init_module);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 139, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "_gldraw_es2.pyx":140
 *     if debug & DEBUG_MSGEXT:
 *         print('init module:', __name__)
 *         print('  from package:', __package__)             # <<<<<<<<<<<<<<
 *         print('  compiled:', __compiled)
 *         print('  GL-type: es2') #px+
 */
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_package); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 140, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 140, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_kp_u_from_package);
    __Pyx_GIVEREF(__pyx_kp_u_from_package);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_kp_u_from_package);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 140, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "_gldraw_es2.pyx":141
 *         print('init module:', __name__)
 *         print('  from package:', __package__)
 *         print('  compiled:', __compiled)             # <<<<<<<<<<<<<<
 *         print('  GL-type: es2') #px+
 *     cube.number_blocks = 0
 */
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_compiled_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 141, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 141, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_kp_u_compiled);
    __Pyx_GIVEREF(__pyx_kp_u_compiled);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_kp_u_compiled);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 141, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "_gldraw_es2.pyx":142
 *         print('  from package:', __package__)
 *         print('  compiled:', __compiled)
 *         print('  GL-type: es2') #px+             # <<<<<<<<<<<<<<
 *     cube.number_blocks = 0
 *     cube.cnt_pick = 0
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 142, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "_gldraw_es2.pyx":138
 * ##pxm-FUNC PD
 * cdef void init_gldraw():
 *     if debug & DEBUG_MSGEXT:             # <<<<<<<<<<<<<<
 *         print('init module:', __name__)
 *         print('  from package:', __package__)
 */
  }

  /* "_gldraw_es2.pyx":143
 *         print('  compiled:', __compiled)
 *         print('  GL-type: es2') #px+
 *     cube.number_blocks = 0             # <<<<<<<<<<<<<<
 *     cube.cnt_pick = 0
 *     cube.idx_debug = 0
 */
  __pyx_v_11_gldraw_es2_cube.number_blocks = 0;

  /* "_gldraw_es2.pyx":144
 *         print('  GL-type: es2') #px+
 *     cube.number_blocks = 0
 *     cube.cnt_pick = 0             # <<<<<<<<<<<<<<
 *     cube.idx_debug = 0
 *     cube.cnt_debug = 0
 */
  __pyx_v_11_gldraw_es2_cube.cnt_pick = 0;

  /* "_gldraw_es2.pyx":145
 *     cube.number_blocks = 0
 *     cube.cnt_pick = 0
 *     cube.idx_debug = 0             # <<<<<<<<<<<<<<
 *     cube.cnt_debug = 0
 *     cube.vertexdatalen = 0
 */
  __pyx_v_11_gldraw_es2_cube.idx_debug = 0;

  /* "_gldraw_es2.pyx":146
 *     cube.cnt_pick = 0
 *     cube.idx_debug = 0
 *     cube.cnt_debug = 0             # <<<<<<<<<<<<<<
 *     cube.vertexdatalen = 0
 *     cube.vertexdata = NULL
 */
  __pyx_v_11_gldraw_es2_cube.cnt_debug = 0;

  /* "_gldraw_es2.pyx":147
 *     cube.idx_debug = 0
 *     cube.cnt_debug = 0
 *     cube.vertexdatalen = 0             # <<<<<<<<<<<<<<
 *     cube.vertexdata = NULL
 * 
 */
  __pyx_v_11_gldraw_es2_cube.vertexdatalen = 0;

  /* "_gldraw_es2.pyx":148
 *     cube.cnt_debug = 0
 *     cube.vertexdatalen = 0
 *     cube.vertexdata = NULL             # <<<<<<<<<<<<<<
 * 
 * ##pxm-FUNC PD nogil
 */
  __pyx_v_11_gldraw_es2_cube.vertexdata = NULL;

  /* "_gldraw_es2.pyx":137
 * 
 * ##pxm-FUNC PD
 * cdef void init_gldraw():             # <<<<<<<<<<<<<<
 *     if debug & DEBUG_MSGEXT:
 *         print('init module:', __name__)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_WriteUnraisable("_gldraw_es2.init_gldraw", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "_gldraw_es2.pyx":151
 * 
 * ##pxm-FUNC PD nogil
 * cdef void sync_block_transformations(short *blocks) nogil:             # <<<<<<<<<<<<<<
 *     cdef unsigned int i #px+
 *     for i in range(cube.number_blocks):
 */

static void __pyx_f_11_gldraw_es2_sync_block_transformations(short *__pyx_v_blocks) {
  unsigned int __pyx_v_i;
  unsigned int __pyx_t_1;
  unsigned int __pyx_t_2;

  /* "_gldraw_es2.pyx":153
 * cdef void sync_block_transformations(short *blocks) nogil:
 *     cdef unsigned int i #px+
 *     for i in range(cube.number_blocks):             # <<<<<<<<<<<<<<
 *         cube.blocks[i].transformation = cube.transformations[blocks[i]]
 *         cube.blocks[i].in_motion = False
 */
  __pyx_t_1 = __pyx_v_11_gldraw_es2_cube.number_blocks;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "_gldraw_es2.pyx":154
 *     cdef unsigned int i #px+
 *     for i in range(cube.number_blocks):
 *         cube.blocks[i].transformation = cube.transformations[blocks[i]]             # <<<<<<<<<<<<<<
 *         cube.blocks[i].in_motion = False
 * 
 */
    (__pyx_v_11_gldraw_es2_cube.blocks[__pyx_v_i]).transformation = (__pyx_v_11_gldraw_es2_cube.transformations[(__pyx_v_blocks[__pyx_v_i])]);

    /* "_gldraw_es2.pyx":155
 *     for i in range(cube.number_blocks):
 *         cube.blocks[i].transformation = cube.transformations[blocks[i]]
 *         cube.blocks[i].in_motion = False             # <<<<<<<<<<<<<<
 * 
 * ##pxm-FUNC PD nogil
 */
    (__pyx_v_11_gldraw_es2_cube.blocks[__pyx_v_i]).in_motion = 0;
  }

  /* "_gldraw_es2.pyx":151
 * 
 * ##pxm-FUNC PD nogil
 * cdef void sync_block_transformations(short *blocks) nogil:             # <<<<<<<<<<<<<<
 *     cdef unsigned int i #px+
 *     for i in range(cube.number_blocks):
 */

  /* function exit code */
}

/* "_gldraw_es2.pyx":158
 * 
 * ##pxm-FUNC PD nogil
 * cdef void sync_animation_start(int blocks_count, short *blocks) nogil:             # <<<<<<<<<<<<<<
 *     matrix_set_identity(cube.rotation_matrix)
 *     cdef int i #px+
 */

static void __pyx_f_11_gldraw_es2_sync_animation_start(int __pyx_v_blocks_count, short *__pyx_v_blocks) {
  int __pyx_v_i;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "_gldraw_es2.pyx":159
 * ##pxm-FUNC PD nogil
 * cdef void sync_animation_start(int blocks_count, short *blocks) nogil:
 *     matrix_set_identity(cube.rotation_matrix)             # <<<<<<<<<<<<<<
 *     cdef int i #px+
 *     for i in range(blocks_count):
 */
  __pyx_f_11_gldraw_es2_matrix_set_identity(__pyx_v_11_gldraw_es2_cube.rotation_matrix);

  /* "_gldraw_es2.pyx":161
 *     matrix_set_identity(cube.rotation_matrix)
 *     cdef int i #px+
 *     for i in range(blocks_count):             # <<<<<<<<<<<<<<
 *         cube.blocks[blocks[i]].in_motion = True
 * 
 */
  __pyx_t_1 = __pyx_v_blocks_count;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "_gldraw_es2.pyx":162
 *     cdef int i #px+
 *     for i in range(blocks_count):
 *         cube.blocks[blocks[i]].in_motion = True             # <<<<<<<<<<<<<<
 * 
 * ##pxm-FUNC PD nogil
 */
    (__pyx_v_11_gldraw_es2_cube.blocks[(__pyx_v_blocks[__pyx_v_i])]).in_motion = 1;
  }

  /* "_gldraw_es2.pyx":158
 * 
 * ##pxm-FUNC PD nogil
 * cdef void sync_animation_start(int blocks_count, short *blocks) nogil:             # <<<<<<<<<<<<<<
 *     matrix_set_identity(cube.rotation_matrix)
 *     cdef int i #px+
 */

  /* function exit code */
}

/* "_gldraw_es2.pyx":165
 * 
 * ##pxm-FUNC PD nogil
 * cdef void sync_animation_next(float angle, float rotation_x, float rotation_y, float rotation_z) nogil:             # <<<<<<<<<<<<<<
 *     angle = angle / 180. * M_PI #px/
 *     #angle = radians(angle)
 */

static void __pyx_f_11_gldraw_es2_sync_animation_next(float __pyx_v_angle, float __pyx_v_rotation_x, float __pyx_v_rotation_y, float __pyx_v_rotation_z) {
  float __pyx_v_x;
  float __pyx_v_y;
  float __pyx_v_z;
  float __pyx_v_c;
  float __pyx_v_s;

  /* "_gldraw_es2.pyx":166
 * ##pxm-FUNC PD nogil
 * cdef void sync_animation_next(float angle, float rotation_x, float rotation_y, float rotation_z) nogil:
 *     angle = angle / 180. * M_PI #px/             # <<<<<<<<<<<<<<
 *     #angle = radians(angle)
 *     cdef float x, y, z #px+
 */
  __pyx_v_angle = ((((double)__pyx_v_angle) / 180.) * M_PI);

  /* "_gldraw_es2.pyx":169
 *     #angle = radians(angle)
 *     cdef float x, y, z #px+
 *     x = rotation_x             # <<<<<<<<<<<<<<
 *     y = rotation_y
 *     z = rotation_z
 */
  __pyx_v_x = __pyx_v_rotation_x;

  /* "_gldraw_es2.pyx":170
 *     cdef float x, y, z #px+
 *     x = rotation_x
 *     y = rotation_y             # <<<<<<<<<<<<<<
 *     z = rotation_z
 *     cdef float c, s #px+
 */
  __pyx_v_y = __pyx_v_rotation_y;

  /* "_gldraw_es2.pyx":171
 *     x = rotation_x
 *     y = rotation_y
 *     z = rotation_z             # <<<<<<<<<<<<<<
 *     cdef float c, s #px+
 *     c = cos(angle)
 */
  __pyx_v_z = __pyx_v_rotation_z;

  /* "_gldraw_es2.pyx":173
 *     z = rotation_z
 *     cdef float c, s #px+
 *     c = cos(angle)             # <<<<<<<<<<<<<<
 *     s = sin(angle)
 * 
 */
  __pyx_v_c = cos(__pyx_v_angle);

  /* "_gldraw_es2.pyx":174
 *     cdef float c, s #px+
 *     c = cos(angle)
 *     s = sin(angle)             # <<<<<<<<<<<<<<
 * 
 *     cube.rotation_matrix[0][0] = x*x*(1-c) + c
 */
  __pyx_v_s = sin(__pyx_v_angle);

  /* "_gldraw_es2.pyx":176
 *     s = sin(angle)
 * 
 *     cube.rotation_matrix[0][0] = x*x*(1-c) + c             # <<<<<<<<<<<<<<
 *     cube.rotation_matrix[0][1] = x*y*(1-c) + z*s
 *     cube.rotation_matrix[0][2] = x*z*(1-c) - y*s
 */
  ((__pyx_v_11_gldraw_es2_cube.rotation_matrix[0])[0]) = (((__pyx_v_x * __pyx_v_x) * (1.0 - __pyx_v_c)) + __pyx_v_c);

  /* "_gldraw_es2.pyx":177
 * 
 *     cube.rotation_matrix[0][0] = x*x*(1-c) + c
 *     cube.rotation_matrix[0][1] = x*y*(1-c) + z*s             # <<<<<<<<<<<<<<
 *     cube.rotation_matrix[0][2] = x*z*(1-c) - y*s
 *     cube.rotation_matrix[1][0] = y*x*(1-c) - z*s
 */
  ((__pyx_v_11_gldraw_es2_cube.rotation_matrix[0])[1]) = (((__pyx_v_x * __pyx_v_y) * (1.0 - __pyx_v_c)) + (__pyx_v_z * __pyx_v_s));

  /* "_gldraw_es2.pyx":178
 *     cube.rotation_matrix[0][0] = x*x*(1-c) + c
 *     cube.rotation_matrix[0][1] = x*y*(1-c) + z*s
 *     cube.rotation_matrix[0][2] = x*z*(1-c) - y*s             # <<<<<<<<<<<<<<
 *     cube.rotation_matrix[1][0] = y*x*(1-c) - z*s
 *     cube.rotation_matrix[1][1] = y*y*(1-c) + c
 */
  ((__pyx_v_11_gldraw_es2_cube.rotation_matrix[0])[2]) = (((__pyx_v_x * __pyx_v_z) * (1.0 - __pyx_v_c)) - (__pyx_v_y * __pyx_v_s));

  /* "_gldraw_es2.pyx":179
 *     cube.rotation_matrix[0][1] = x*y*(1-c) + z*s
 *     cube.rotation_matrix[0][2] = x*z*(1-c) - y*s
 *     cube.rotation_matrix[1][0] = y*x*(1-c) - z*s             # <<<<<<<<<<<<<<
 *     cube.rotation_matrix[1][1] = y*y*(1-c) + c
 *     cube.rotation_matrix[1][2] = y*z*(1-c) + x*s
 */
  ((__pyx_v_11_gldraw_es2_cube.rotation_matrix[1])[0]) = (((__pyx_v_y * __pyx_v_x) * (1.0 - __pyx_v_c)) - (__pyx_v_z * __pyx_v_s));

  /* "_gldraw_es2.pyx":180
 *     cube.rotation_matrix[0][2] = x*z*(1-c) - y*s
 *     cube.rotation_matrix[1][0] = y*x*(1-c) - z*s
 *     cube.rotation_matrix[1][1] = y*y*(1-c) + c             # <<<<<<<<<<<<<<
 *     cube.rotation_matrix[1][2] = y*z*(1-c) + x*s
 *     cube.rotation_matrix[2][0] = x*z*(1-c) + y*s
 */
  ((__pyx_v_11_gldraw_es2_cube.rotation_matrix[1])[1]) = (((__pyx_v_y * __pyx_v_y) * (1.0 - __pyx_v_c)) + __pyx_v_c);

  /* "_gldraw_es2.pyx":181
 *     cube.rotation_matrix[1][0] = y*x*(1-c) - z*s
 *     cube.rotation_matrix[1][1] = y*y*(1-c) + c
 *     cube.rotation_matrix[1][2] = y*z*(1-c) + x*s             # <<<<<<<<<<<<<<
 *     cube.rotation_matrix[2][0] = x*z*(1-c) + y*s
 *     cube.rotation_matrix[2][1] = y*z*(1-c) - x*s
 */
  ((__pyx_v_11_gldraw_es2_cube.rotation_matrix[1])[2]) = (((__pyx_v_y * __pyx_v_z) * (1.0 - __pyx_v_c)) + (__pyx_v_x * __pyx_v_s));

  /* "_gldraw_es2.pyx":182
 *     cube.rotation_matrix[1][1] = y*y*(1-c) + c
 *     cube.rotation_matrix[1][2] = y*z*(1-c) + x*s
 *     cube.rotation_matrix[2][0] = x*z*(1-c) + y*s             # <<<<<<<<<<<<<<
 *     cube.rotation_matrix[2][1] = y*z*(1-c) - x*s
 *     cube.rotation_matrix[2][2] = z*z*(1-c) + c
 */
  ((__pyx_v_11_gldraw_es2_cube.rotation_matrix[2])[0]) = (((__pyx_v_x * __pyx_v_z) * (1.0 - __pyx_v_c)) + (__pyx_v_y * __pyx_v_s));

  /* "_gldraw_es2.pyx":183
 *     cube.rotation_matrix[1][2] = y*z*(1-c) + x*s
 *     cube.rotation_matrix[2][0] = x*z*(1-c) + y*s
 *     cube.rotation_matrix[2][1] = y*z*(1-c) - x*s             # <<<<<<<<<<<<<<
 *     cube.rotation_matrix[2][2] = z*z*(1-c) + c
 * 
 */
  ((__pyx_v_11_gldraw_es2_cube.rotation_matrix[2])[1]) = (((__pyx_v_y * __pyx_v_z) * (1.0 - __pyx_v_c)) - (__pyx_v_x * __pyx_v_s));

  /* "_gldraw_es2.pyx":184
 *     cube.rotation_matrix[2][0] = x*z*(1-c) + y*s
 *     cube.rotation_matrix[2][1] = y*z*(1-c) - x*s
 *     cube.rotation_matrix[2][2] = z*z*(1-c) + c             # <<<<<<<<<<<<<<
 * 
 * ##pxm-FUNC P nogil
 */
  ((__pyx_v_11_gldraw_es2_cube.rotation_matrix[2])[2]) = (((__pyx_v_z * __pyx_v_z) * (1.0 - __pyx_v_c)) + __pyx_v_c);

  /* "_gldraw_es2.pyx":165
 * 
 * ##pxm-FUNC PD nogil
 * cdef void sync_animation_next(float angle, float rotation_x, float rotation_y, float rotation_z) nogil:             # <<<<<<<<<<<<<<
 *     angle = angle / 180. * M_PI #px/
 *     #angle = radians(angle)
 */

  /* function exit code */
}

/* "_gldraw_es2.pyx":187
 * 
 * ##pxm-FUNC P nogil
 * cdef void matrix_mult(mat4 &dest, mat4 &src1, mat4 &src2) nogil:             # <<<<<<<<<<<<<<
 *     cdef int i, j, k #px+
 *     cdef float sum_ #px+
 */

static void __pyx_f_11_gldraw_es2_matrix_mult(__pyx_t_11_gldraw_es2_vec4 *__pyx_v_dest, __pyx_t_11_gldraw_es2_vec4 *__pyx_v_src1, __pyx_t_11_gldraw_es2_vec4 *__pyx_v_src2) {
  int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_v_k;
  float __pyx_v_sum_;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;

  /* "_gldraw_es2.pyx":190
 *     cdef int i, j, k #px+
 *     cdef float sum_ #px+
 *     for j in range(4):             # <<<<<<<<<<<<<<
 *         for i in range(4):
 *             sum_ = 0.
 */
  for (__pyx_t_1 = 0; __pyx_t_1 < 4; __pyx_t_1+=1) {
    __pyx_v_j = __pyx_t_1;

    /* "_gldraw_es2.pyx":191
 *     cdef float sum_ #px+
 *     for j in range(4):
 *         for i in range(4):             # <<<<<<<<<<<<<<
 *             sum_ = 0.
 *             for k in range(4):
 */
    for (__pyx_t_2 = 0; __pyx_t_2 < 4; __pyx_t_2+=1) {
      __pyx_v_i = __pyx_t_2;

      /* "_gldraw_es2.pyx":192
 *     for j in range(4):
 *         for i in range(4):
 *             sum_ = 0.             # <<<<<<<<<<<<<<
 *             for k in range(4):
 *                 sum_ += src1[k][i] * src2[j][k]
 */
      __pyx_v_sum_ = 0.;

      /* "_gldraw_es2.pyx":193
 *         for i in range(4):
 *             sum_ = 0.
 *             for k in range(4):             # <<<<<<<<<<<<<<
 *                 sum_ += src1[k][i] * src2[j][k]
 *             dest[j][i] = sum_
 */
      for (__pyx_t_3 = 0; __pyx_t_3 < 4; __pyx_t_3+=1) {
        __pyx_v_k = __pyx_t_3;

        /* "_gldraw_es2.pyx":194
 *             sum_ = 0.
 *             for k in range(4):
 *                 sum_ += src1[k][i] * src2[j][k]             # <<<<<<<<<<<<<<
 *             dest[j][i] = sum_
 * 
 */
        __pyx_v_sum_ = (__pyx_v_sum_ + (((__pyx_v_src1[__pyx_v_k])[__pyx_v_i]) * ((__pyx_v_src2[__pyx_v_j])[__pyx_v_k])));
      }

      /* "_gldraw_es2.pyx":195
 *             for k in range(4):
 *                 sum_ += src1[k][i] * src2[j][k]
 *             dest[j][i] = sum_             # <<<<<<<<<<<<<<
 * 
 * ##pxm-FUNC PD nogil
 */
      ((__pyx_v_dest[__pyx_v_j])[__pyx_v_i]) = __pyx_v_sum_;
    }
  }

  /* "_gldraw_es2.pyx":187
 * 
 * ##pxm-FUNC P nogil
 * cdef void matrix_mult(mat4 &dest, mat4 &src1, mat4 &src2) nogil:             # <<<<<<<<<<<<<<
 *     cdef int i, j, k #px+
 *     cdef float sum_ #px+
 */

  /* function exit code */
}

/* "_gldraw_es2.pyx":198
 * 
 * ##pxm-FUNC PD nogil
 * cdef void gl_draw_cube() nogil:             # <<<<<<<<<<<<<<
 *     cdef unsigned int i #px+
 *     cdef mat4 object_matrix #px/
 */

static void __pyx_f_11_gldraw_es2_gl_draw_cube(void) {
  unsigned int __pyx_v_i;
  __pyx_t_11_gldraw_es2_mat4 __pyx_v_object_matrix;
  unsigned int __pyx_t_1;
  unsigned int __pyx_t_2;
  int __pyx_t_3;

  /* "_gldraw_es2.pyx":202
 *     cdef mat4 object_matrix #px/
 *     #object_matrix = mat4()
 *     for i in range(cube.number_blocks):             # <<<<<<<<<<<<<<
 *         if cube.blocks[i].in_motion:
 *             matrix_mult(object_matrix, cube.rotation_matrix, cube.blocks[i].transformation)
 */
  __pyx_t_1 = __pyx_v_11_gldraw_es2_cube.number_blocks;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "_gldraw_es2.pyx":203
 *     #object_matrix = mat4()
 *     for i in range(cube.number_blocks):
 *         if cube.blocks[i].in_motion:             # <<<<<<<<<<<<<<
 *             matrix_mult(object_matrix, cube.rotation_matrix, cube.blocks[i].transformation)
 *             glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, &object_matrix[0][0]) #px/
 */
    __pyx_t_3 = ((__pyx_v_11_gldraw_es2_cube.blocks[__pyx_v_i]).in_motion != 0);
    if (__pyx_t_3) {

      /* "_gldraw_es2.pyx":204
 *     for i in range(cube.number_blocks):
 *         if cube.blocks[i].in_motion:
 *             matrix_mult(object_matrix, cube.rotation_matrix, cube.blocks[i].transformation)             # <<<<<<<<<<<<<<
 *             glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, &object_matrix[0][0]) #px/
 *             #glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, object_matrix)
 */
      __pyx_f_11_gldraw_es2_matrix_mult(__pyx_v_object_matrix, __pyx_v_11_gldraw_es2_cube.rotation_matrix, (__pyx_v_11_gldraw_es2_cube.blocks[__pyx_v_i]).transformation);

      /* "_gldraw_es2.pyx":205
 *         if cube.blocks[i].in_motion:
 *             matrix_mult(object_matrix, cube.rotation_matrix, cube.blocks[i].transformation)
 *             glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, &object_matrix[0][0]) #px/             # <<<<<<<<<<<<<<
 *             #glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, object_matrix)
 *         else:
 */
      glUniformMatrix4fv(__pyx_v_11_gldraw_es2_cube.object_location, 1, GL_FALSE, (&((__pyx_v_object_matrix[0])[0])));

      /* "_gldraw_es2.pyx":203
 *     #object_matrix = mat4()
 *     for i in range(cube.number_blocks):
 *         if cube.blocks[i].in_motion:             # <<<<<<<<<<<<<<
 *             matrix_mult(object_matrix, cube.rotation_matrix, cube.blocks[i].transformation)
 *             glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, &object_matrix[0][0]) #px/
 */
      goto __pyx_L5;
    }

    /* "_gldraw_es2.pyx":208
 *             #glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, object_matrix)
 *         else:
 *             glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, &cube.blocks[i].transformation[0][0]) #px/             # <<<<<<<<<<<<<<
 *             #glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, cube.blocks[i].transformation)
 *         glDrawArrays(GL_TRIANGLES, cube.blocks[i].idx_triangles, cube.blocks[i].cnt_triangles)
 */
    /*else*/ {
      glUniformMatrix4fv(__pyx_v_11_gldraw_es2_cube.object_location, 1, GL_FALSE, (&(((__pyx_v_11_gldraw_es2_cube.blocks[__pyx_v_i]).transformation[0])[0])));
    }
    __pyx_L5:;

    /* "_gldraw_es2.pyx":210
 *             glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, &cube.blocks[i].transformation[0][0]) #px/
 *             #glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, cube.blocks[i].transformation)
 *         glDrawArrays(GL_TRIANGLES, cube.blocks[i].idx_triangles, cube.blocks[i].cnt_triangles)             # <<<<<<<<<<<<<<
 * 
 * ##pxm-FUNC PD nogil
 */
    glDrawArrays(GL_TRIANGLES, (__pyx_v_11_gldraw_es2_cube.blocks[__pyx_v_i]).idx_triangles, (__pyx_v_11_gldraw_es2_cube.blocks[__pyx_v_i]).cnt_triangles);
  }

  /* "_gldraw_es2.pyx":198
 * 
 * ##pxm-FUNC PD nogil
 * cdef void gl_draw_cube() nogil:             # <<<<<<<<<<<<<<
 *     cdef unsigned int i #px+
 *     cdef mat4 object_matrix #px/
 */

  /* function exit code */
}

/* "_gldraw_es2.pyx":213
 * 
 * ##pxm-FUNC PD nogil
 * cdef void gl_pick_cube() nogil:             # <<<<<<<<<<<<<<
 *     glDrawArrays(GL_TRIANGLES, 0, cube.cnt_pick)
 * 
 */

static void __pyx_f_11_gldraw_es2_gl_pick_cube(void) {

  /* "_gldraw_es2.pyx":214
 * ##pxm-FUNC PD nogil
 * cdef void gl_pick_cube() nogil:
 *     glDrawArrays(GL_TRIANGLES, 0, cube.cnt_pick)             # <<<<<<<<<<<<<<
 * 
 * ##pxm-FUNC PD nogil
 */
  glDrawArrays(GL_TRIANGLES, 0, __pyx_v_11_gldraw_es2_cube.cnt_pick);

  /* "_gldraw_es2.pyx":213
 * 
 * ##pxm-FUNC PD nogil
 * cdef void gl_pick_cube() nogil:             # <<<<<<<<<<<<<<
 *     glDrawArrays(GL_TRIANGLES, 0, cube.cnt_pick)
 * 
 */

  /* function exit code */
}

/* "_gldraw_es2.pyx":217
 * 
 * ##pxm-FUNC PD nogil
 * cdef void gl_init_buffers() nogil:             # <<<<<<<<<<<<<<
 *     glGenBuffers(1, &cube.glbuffer) #px/
 *     #cube.glbuffer = glGenBuffers(1)
 */

static void __pyx_f_11_gldraw_es2_gl_init_buffers(void) {

  /* "_gldraw_es2.pyx":218
 * ##pxm-FUNC PD nogil
 * cdef void gl_init_buffers() nogil:
 *     glGenBuffers(1, &cube.glbuffer) #px/             # <<<<<<<<<<<<<<
 *     #cube.glbuffer = glGenBuffers(1)
 * 
 */
  glGenBuffers(1, (&__pyx_v_11_gldraw_es2_cube.glbuffer));

  /* "_gldraw_es2.pyx":217
 * 
 * ##pxm-FUNC PD nogil
 * cdef void gl_init_buffers() nogil:             # <<<<<<<<<<<<<<
 *     glGenBuffers(1, &cube.glbuffer) #px/
 *     #cube.glbuffer = glGenBuffers(1)
 */

  /* function exit code */
}

/* "_gldraw_es2.pyx":222
 * 
 * ##pxm-FUNC PD nogil
 * cdef void gl_delete_buffers() nogil:             # <<<<<<<<<<<<<<
 *     glBindBuffer(GL_ARRAY_BUFFER, 0)
 *     glDeleteBuffers(1, &cube.glbuffer) #px/
 */

static void __pyx_f_11_gldraw_es2_gl_delete_buffers(void) {

  /* "_gldraw_es2.pyx":223
 * ##pxm-FUNC PD nogil
 * cdef void gl_delete_buffers() nogil:
 *     glBindBuffer(GL_ARRAY_BUFFER, 0)             # <<<<<<<<<<<<<<
 *     glDeleteBuffers(1, &cube.glbuffer) #px/
 *     #glDeleteBuffers(1, [cube.glbuffer])
 */
  glBindBuffer(GL_ARRAY_BUFFER, 0);

  /* "_gldraw_es2.pyx":224
 * cdef void gl_delete_buffers() nogil:
 *     glBindBuffer(GL_ARRAY_BUFFER, 0)
 *     glDeleteBuffers(1, &cube.glbuffer) #px/             # <<<<<<<<<<<<<<
 *     #glDeleteBuffers(1, [cube.glbuffer])
 *     cube.glbuffer = 0
 */
  glDeleteBuffers(1, (&__pyx_v_11_gldraw_es2_cube.glbuffer));

  /* "_gldraw_es2.pyx":226
 *     glDeleteBuffers(1, &cube.glbuffer) #px/
 *     #glDeleteBuffers(1, [cube.glbuffer])
 *     cube.glbuffer = 0             # <<<<<<<<<<<<<<
 * 
 * ##pxm-FUNC P nogil
 */
  __pyx_v_11_gldraw_es2_cube.glbuffer = 0;

  /* "_gldraw_es2.pyx":222
 * 
 * ##pxm-FUNC PD nogil
 * cdef void gl_delete_buffers() nogil:             # <<<<<<<<<<<<<<
 *     glBindBuffer(GL_ARRAY_BUFFER, 0)
 *     glDeleteBuffers(1, &cube.glbuffer) #px/
 */

  /* function exit code */
}

/* "_gldraw_es2.pyx":229
 * 
 * ##pxm-FUNC P nogil
 * cdef void _gl_enable_pointer(GLuint index, GLint size, GLenum type, GLboolean normalized, long pointer) nogil:             # <<<<<<<<<<<<<<
 *     glVertexAttribPointer(index, size, type, normalized, 0, <void*>pointer) #px/
 *     #glVertexAttribPointer(index, size, type, normalized, 0, ctypes.cast(pointer, ctypes.c_void_p))
 */

static void __pyx_f_11_gldraw_es2__gl_enable_pointer(GLuint __pyx_v_index, GLint __pyx_v_size, GLenum __pyx_v_type, GLboolean __pyx_v_normalized, long __pyx_v_pointer) {

  /* "_gldraw_es2.pyx":230
 * ##pxm-FUNC P nogil
 * cdef void _gl_enable_pointer(GLuint index, GLint size, GLenum type, GLboolean normalized, long pointer) nogil:
 *     glVertexAttribPointer(index, size, type, normalized, 0, <void*>pointer) #px/             # <<<<<<<<<<<<<<
 *     #glVertexAttribPointer(index, size, type, normalized, 0, ctypes.cast(pointer, ctypes.c_void_p))
 *     glEnableVertexAttribArray(index)
 */
  glVertexAttribPointer(__pyx_v_index, __pyx_v_size, __pyx_v_type, __pyx_v_normalized, 0, ((void *)__pyx_v_pointer));

  /* "_gldraw_es2.pyx":232
 *     glVertexAttribPointer(index, size, type, normalized, 0, <void*>pointer) #px/
 *     #glVertexAttribPointer(index, size, type, normalized, 0, ctypes.cast(pointer, ctypes.c_void_p))
 *     glEnableVertexAttribArray(index)             # <<<<<<<<<<<<<<
 * 
 * ##pxm-FUNC P nogil
 */
  glEnableVertexAttribArray(__pyx_v_index);

  /* "_gldraw_es2.pyx":229
 * 
 * ##pxm-FUNC P nogil
 * cdef void _gl_enable_pointer(GLuint index, GLint size, GLenum type, GLboolean normalized, long pointer) nogil:             # <<<<<<<<<<<<<<
 *     glVertexAttribPointer(index, size, type, normalized, 0, <void*>pointer) #px/
 *     #glVertexAttribPointer(index, size, type, normalized, 0, ctypes.cast(pointer, ctypes.c_void_p))
 */

  /* function exit code */
}

/* "_gldraw_es2.pyx":235
 * 
 * ##pxm-FUNC P nogil
 * cdef void _gl_disable_pointer(GLuint index) nogil:             # <<<<<<<<<<<<<<
 *     glVertexAttribPointer(index, 4, GL_FLOAT, GL_FALSE, 0, NULL) #px/
 *     #glVertexAttribPointer(index, 4, GL_FLOAT, GL_FALSE, 0, ctypes.cast(0, ctypes.c_void_p))
 */

static void __pyx_f_11_gldraw_es2__gl_disable_pointer(GLuint __pyx_v_index) {

  /* "_gldraw_es2.pyx":236
 * ##pxm-FUNC P nogil
 * cdef void _gl_disable_pointer(GLuint index) nogil:
 *     glVertexAttribPointer(index, 4, GL_FLOAT, GL_FALSE, 0, NULL) #px/             # <<<<<<<<<<<<<<
 *     #glVertexAttribPointer(index, 4, GL_FLOAT, GL_FALSE, 0, ctypes.cast(0, ctypes.c_void_p))
 *     glDisableVertexAttribArray(index)
 */
  glVertexAttribPointer(__pyx_v_index, 4, GL_FLOAT, GL_FALSE, 0, NULL);

  /* "_gldraw_es2.pyx":238
 *     glVertexAttribPointer(index, 4, GL_FLOAT, GL_FALSE, 0, NULL) #px/
 *     #glVertexAttribPointer(index, 4, GL_FLOAT, GL_FALSE, 0, ctypes.cast(0, ctypes.c_void_p))
 *     glDisableVertexAttribArray(index)             # <<<<<<<<<<<<<<
 * 
 * ##pxm-FUNC PD nogil
 */
  glDisableVertexAttribArray(__pyx_v_index);

  /* "_gldraw_es2.pyx":235
 * 
 * ##pxm-FUNC P nogil
 * cdef void _gl_disable_pointer(GLuint index) nogil:             # <<<<<<<<<<<<<<
 *     glVertexAttribPointer(index, 4, GL_FLOAT, GL_FALSE, 0, NULL) #px/
 *     #glVertexAttribPointer(index, 4, GL_FLOAT, GL_FALSE, 0, ctypes.cast(0, ctypes.c_void_p))
 */

  /* function exit code */
}

/* "_gldraw_es2.pyx":241
 * 
 * ##pxm-FUNC PD nogil
 * cdef void sync_blocks(int nblocks, short *cnts_block, int idx_debug, int cnt_debug, int cnt_pick) nogil:             # <<<<<<<<<<<<<<
 *     cdef unsigned int idx_block, idx #px+
 *     cube.number_blocks = nblocks
 */

static void __pyx_f_11_gldraw_es2_sync_blocks(int __pyx_v_nblocks, short *__pyx_v_cnts_block, int __pyx_v_idx_debug, int __pyx_v_cnt_debug, int __pyx_v_cnt_pick) {
  unsigned int __pyx_v_idx_block;
  unsigned int __pyx_v_idx;
  unsigned int __pyx_t_1;
  unsigned int __pyx_t_2;

  /* "_gldraw_es2.pyx":243
 * cdef void sync_blocks(int nblocks, short *cnts_block, int idx_debug, int cnt_debug, int cnt_pick) nogil:
 *     cdef unsigned int idx_block, idx #px+
 *     cube.number_blocks = nblocks             # <<<<<<<<<<<<<<
 * 
 *     idx_block = 0
 */
  __pyx_v_11_gldraw_es2_cube.number_blocks = __pyx_v_nblocks;

  /* "_gldraw_es2.pyx":245
 *     cube.number_blocks = nblocks
 * 
 *     idx_block = 0             # <<<<<<<<<<<<<<
 *     for idx in range(cube.number_blocks):
 *         cube.blocks[idx].idx_triangles = idx_block
 */
  __pyx_v_idx_block = 0;

  /* "_gldraw_es2.pyx":246
 * 
 *     idx_block = 0
 *     for idx in range(cube.number_blocks):             # <<<<<<<<<<<<<<
 *         cube.blocks[idx].idx_triangles = idx_block
 *         cube.blocks[idx].cnt_triangles = cnts_block[idx]
 */
  __pyx_t_1 = __pyx_v_11_gldraw_es2_cube.number_blocks;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_idx = __pyx_t_2;

    /* "_gldraw_es2.pyx":247
 *     idx_block = 0
 *     for idx in range(cube.number_blocks):
 *         cube.blocks[idx].idx_triangles = idx_block             # <<<<<<<<<<<<<<
 *         cube.blocks[idx].cnt_triangles = cnts_block[idx]
 *         idx_block += cnts_block[idx]
 */
    (__pyx_v_11_gldraw_es2_cube.blocks[__pyx_v_idx]).idx_triangles = __pyx_v_idx_block;

    /* "_gldraw_es2.pyx":248
 *     for idx in range(cube.number_blocks):
 *         cube.blocks[idx].idx_triangles = idx_block
 *         cube.blocks[idx].cnt_triangles = cnts_block[idx]             # <<<<<<<<<<<<<<
 *         idx_block += cnts_block[idx]
 * 
 */
    (__pyx_v_11_gldraw_es2_cube.blocks[__pyx_v_idx]).cnt_triangles = (__pyx_v_cnts_block[__pyx_v_idx]);

    /* "_gldraw_es2.pyx":249
 *         cube.blocks[idx].idx_triangles = idx_block
 *         cube.blocks[idx].cnt_triangles = cnts_block[idx]
 *         idx_block += cnts_block[idx]             # <<<<<<<<<<<<<<
 * 
 *     cube.cnt_pick = cnt_pick
 */
    __pyx_v_idx_block = (__pyx_v_idx_block + (__pyx_v_cnts_block[__pyx_v_idx]));
  }

  /* "_gldraw_es2.pyx":251
 *         idx_block += cnts_block[idx]
 * 
 *     cube.cnt_pick = cnt_pick             # <<<<<<<<<<<<<<
 *     cube.idx_debug = idx_debug
 *     cube.cnt_debug = cnt_debug
 */
  __pyx_v_11_gldraw_es2_cube.cnt_pick = __pyx_v_cnt_pick;

  /* "_gldraw_es2.pyx":252
 * 
 *     cube.cnt_pick = cnt_pick
 *     cube.idx_debug = idx_debug             # <<<<<<<<<<<<<<
 *     cube.cnt_debug = cnt_debug
 * 
 */
  __pyx_v_11_gldraw_es2_cube.idx_debug = __pyx_v_idx_debug;

  /* "_gldraw_es2.pyx":253
 *     cube.cnt_pick = cnt_pick
 *     cube.idx_debug = idx_debug
 *     cube.cnt_debug = cnt_debug             # <<<<<<<<<<<<<<
 * 
 * ##pxm-FUNC PD nogil
 */
  __pyx_v_11_gldraw_es2_cube.cnt_debug = __pyx_v_cnt_debug;

  /* "_gldraw_es2.pyx":241
 * 
 * ##pxm-FUNC PD nogil
 * cdef void sync_blocks(int nblocks, short *cnts_block, int idx_debug, int cnt_debug, int cnt_pick) nogil:             # <<<<<<<<<<<<<<
 *     cdef unsigned int idx_block, idx #px+
 *     cube.number_blocks = nblocks
 */

  /* function exit code */
}

/* "_gldraw_es2.pyx":256
 * 
 * ##pxm-FUNC PD nogil
 * cdef void sync_vertexdata(int vertexdatalen, char *vertexdata, long *vertexpointers) nogil:             # <<<<<<<<<<<<<<
 *     cube.vertexdatalen = vertexdatalen
 *     cube.vertexdata = vertexdata
 */

static void __pyx_f_11_gldraw_es2_sync_vertexdata(int __pyx_v_vertexdatalen, char *__pyx_v_vertexdata, long *__pyx_v_vertexpointers) {

  /* "_gldraw_es2.pyx":257
 * ##pxm-FUNC PD nogil
 * cdef void sync_vertexdata(int vertexdatalen, char *vertexdata, long *vertexpointers) nogil:
 *     cube.vertexdatalen = vertexdatalen             # <<<<<<<<<<<<<<
 *     cube.vertexdata = vertexdata
 *     cube.attrib_pointers[0] = 0
 */
  __pyx_v_11_gldraw_es2_cube.vertexdatalen = __pyx_v_vertexdatalen;

  /* "_gldraw_es2.pyx":258
 * cdef void sync_vertexdata(int vertexdatalen, char *vertexdata, long *vertexpointers) nogil:
 *     cube.vertexdatalen = vertexdatalen
 *     cube.vertexdata = vertexdata             # <<<<<<<<<<<<<<
 *     cube.attrib_pointers[0] = 0
 *     cube.attrib_pointers[1] = vertexpointers[0]
 */
  __pyx_v_11_gldraw_es2_cube.vertexdata = __pyx_v_vertexdata;

  /* "_gldraw_es2.pyx":259
 *     cube.vertexdatalen = vertexdatalen
 *     cube.vertexdata = vertexdata
 *     cube.attrib_pointers[0] = 0             # <<<<<<<<<<<<<<
 *     cube.attrib_pointers[1] = vertexpointers[0]
 *     cube.attrib_pointers[2] = vertexpointers[1]
 */
  (__pyx_v_11_gldraw_es2_cube.attrib_pointers[0]) = 0;

  /* "_gldraw_es2.pyx":260
 *     cube.vertexdata = vertexdata
 *     cube.attrib_pointers[0] = 0
 *     cube.attrib_pointers[1] = vertexpointers[0]             # <<<<<<<<<<<<<<
 *     cube.attrib_pointers[2] = vertexpointers[1]
 *     cube.attrib_pointers[3] = vertexpointers[2]
 */
  (__pyx_v_11_gldraw_es2_cube.attrib_pointers[1]) = (__pyx_v_vertexpointers[0]);

  /* "_gldraw_es2.pyx":261
 *     cube.attrib_pointers[0] = 0
 *     cube.attrib_pointers[1] = vertexpointers[0]
 *     cube.attrib_pointers[2] = vertexpointers[1]             # <<<<<<<<<<<<<<
 *     cube.attrib_pointers[3] = vertexpointers[2]
 *     cube.attrib_pointers[4] = vertexpointers[3]
 */
  (__pyx_v_11_gldraw_es2_cube.attrib_pointers[2]) = (__pyx_v_vertexpointers[1]);

  /* "_gldraw_es2.pyx":262
 *     cube.attrib_pointers[1] = vertexpointers[0]
 *     cube.attrib_pointers[2] = vertexpointers[1]
 *     cube.attrib_pointers[3] = vertexpointers[2]             # <<<<<<<<<<<<<<
 *     cube.attrib_pointers[4] = vertexpointers[3]
 *     cube.attrib_pointers[5] = vertexpointers[4]
 */
  (__pyx_v_11_gldraw_es2_cube.attrib_pointers[3]) = (__pyx_v_vertexpointers[2]);

  /* "_gldraw_es2.pyx":263
 *     cube.attrib_pointers[2] = vertexpointers[1]
 *     cube.attrib_pointers[3] = vertexpointers[2]
 *     cube.attrib_pointers[4] = vertexpointers[3]             # <<<<<<<<<<<<<<
 *     cube.attrib_pointers[5] = vertexpointers[4]
 *     cube.attrib_pointers[6] = vertexpointers[5]
 */
  (__pyx_v_11_gldraw_es2_cube.attrib_pointers[4]) = (__pyx_v_vertexpointers[3]);

  /* "_gldraw_es2.pyx":264
 *     cube.attrib_pointers[3] = vertexpointers[2]
 *     cube.attrib_pointers[4] = vertexpointers[3]
 *     cube.attrib_pointers[5] = vertexpointers[4]             # <<<<<<<<<<<<<<
 *     cube.attrib_pointers[6] = vertexpointers[5]
 * 
 */
  (__pyx_v_11_gldraw_es2_cube.attrib_pointers[5]) = (__pyx_v_vertexpointers[4]);

  /* "_gldraw_es2.pyx":265
 *     cube.attrib_pointers[4] = vertexpointers[3]
 *     cube.attrib_pointers[5] = vertexpointers[4]
 *     cube.attrib_pointers[6] = vertexpointers[5]             # <<<<<<<<<<<<<<
 * 
 * ##pxm-FUNC PD nogil
 */
  (__pyx_v_11_gldraw_es2_cube.attrib_pointers[6]) = (__pyx_v_vertexpointers[5]);

  /* "_gldraw_es2.pyx":256
 * 
 * ##pxm-FUNC PD nogil
 * cdef void sync_vertexdata(int vertexdatalen, char *vertexdata, long *vertexpointers) nogil:             # <<<<<<<<<<<<<<
 *     cube.vertexdatalen = vertexdatalen
 *     cube.vertexdata = vertexdata
 */

  /* function exit code */
}

/* "_gldraw_es2.pyx":268
 * 
 * ##pxm-FUNC PD nogil
 * cdef void sync_transformations(int transformations_count, mat4 *transformations) nogil:             # <<<<<<<<<<<<<<
 *     cdef int t #px+
 *     cdef unsigned int i, j #px+
 */

static void __pyx_f_11_gldraw_es2_sync_transformations(int __pyx_v_transformations_count, __pyx_t_11_gldraw_es2_mat4 *__pyx_v_transformations) {
  int __pyx_v_t;
  unsigned int __pyx_v_i;
  unsigned int __pyx_v_j;
  int __pyx_t_1;
  int __pyx_t_2;
  unsigned int __pyx_t_3;
  unsigned int __pyx_t_4;

  /* "_gldraw_es2.pyx":271
 *     cdef int t #px+
 *     cdef unsigned int i, j #px+
 *     for t in range(transformations_count):             # <<<<<<<<<<<<<<
 *         for i in range(4):
 *             for j in range(4):
 */
  __pyx_t_1 = __pyx_v_transformations_count;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_t = __pyx_t_2;

    /* "_gldraw_es2.pyx":272
 *     cdef unsigned int i, j #px+
 *     for t in range(transformations_count):
 *         for i in range(4):             # <<<<<<<<<<<<<<
 *             for j in range(4):
 *                 cube.transformations[t][i][j] = float(transformations[t][i][j])
 */
    for (__pyx_t_3 = 0; __pyx_t_3 < 4; __pyx_t_3+=1) {
      __pyx_v_i = __pyx_t_3;

      /* "_gldraw_es2.pyx":273
 *     for t in range(transformations_count):
 *         for i in range(4):
 *             for j in range(4):             # <<<<<<<<<<<<<<
 *                 cube.transformations[t][i][j] = float(transformations[t][i][j])
 * 
 */
      for (__pyx_t_4 = 0; __pyx_t_4 < 4; __pyx_t_4+=1) {
        __pyx_v_j = __pyx_t_4;

        /* "_gldraw_es2.pyx":274
 *         for i in range(4):
 *             for j in range(4):
 *                 cube.transformations[t][i][j] = float(transformations[t][i][j])             # <<<<<<<<<<<<<<
 * 
 * ##pxm-FUNC PD nogil
 */
        (((__pyx_v_11_gldraw_es2_cube.transformations[__pyx_v_t])[__pyx_v_i])[__pyx_v_j]) = ((double)(((__pyx_v_transformations[__pyx_v_t])[__pyx_v_i])[__pyx_v_j]));
      }
    }
  }

  /* "_gldraw_es2.pyx":268
 * 
 * ##pxm-FUNC PD nogil
 * cdef void sync_transformations(int transformations_count, mat4 *transformations) nogil:             # <<<<<<<<<<<<<<
 *     cdef int t #px+
 *     cdef unsigned int i, j #px+
 */

  /* function exit code */
}

/* "_gldraw_es2.pyx":277
 * 
 * ##pxm-FUNC PD nogil
 * cdef void gl_enable_data() nogil:             # <<<<<<<<<<<<<<
 *     glBindBuffer(GL_ARRAY_BUFFER, cube.glbuffer)
 *     glBufferData(GL_ARRAY_BUFFER, cube.vertexdatalen, cube.vertexdata, GL_STATIC_DRAW)
 */

static void __pyx_f_11_gldraw_es2_gl_enable_data(void) {

  /* "_gldraw_es2.pyx":278
 * ##pxm-FUNC PD nogil
 * cdef void gl_enable_data() nogil:
 *     glBindBuffer(GL_ARRAY_BUFFER, cube.glbuffer)             # <<<<<<<<<<<<<<
 *     glBufferData(GL_ARRAY_BUFFER, cube.vertexdatalen, cube.vertexdata, GL_STATIC_DRAW)
 * 
 */
  glBindBuffer(GL_ARRAY_BUFFER, __pyx_v_11_gldraw_es2_cube.glbuffer);

  /* "_gldraw_es2.pyx":279
 * cdef void gl_enable_data() nogil:
 *     glBindBuffer(GL_ARRAY_BUFFER, cube.glbuffer)
 *     glBufferData(GL_ARRAY_BUFFER, cube.vertexdatalen, cube.vertexdata, GL_STATIC_DRAW)             # <<<<<<<<<<<<<<
 * 
 *     _gl_enable_pointer(ATTRIB_LOCATION,   3, GL_FLOAT, GL_FALSE, cube.attrib_pointers[0])
 */
  glBufferData(GL_ARRAY_BUFFER, __pyx_v_11_gldraw_es2_cube.vertexdatalen, __pyx_v_11_gldraw_es2_cube.vertexdata, GL_STATIC_DRAW);

  /* "_gldraw_es2.pyx":281
 *     glBufferData(GL_ARRAY_BUFFER, cube.vertexdatalen, cube.vertexdata, GL_STATIC_DRAW)
 * 
 *     _gl_enable_pointer(ATTRIB_LOCATION,   3, GL_FLOAT, GL_FALSE, cube.attrib_pointers[0])             # <<<<<<<<<<<<<<
 *     _gl_enable_pointer(ATTRIB_LOCATION+1, 3, GL_FLOAT, GL_FALSE, cube.attrib_pointers[1])
 *     _gl_enable_pointer(ATTRIB_LOCATION+2, 3, GL_UNSIGNED_BYTE, GL_TRUE, cube.attrib_pointers[2])
 */
  __pyx_f_11_gldraw_es2__gl_enable_pointer(__pyx_e_11_gldraw_es2_ATTRIB_LOCATION, 3, GL_FLOAT, GL_FALSE, (__pyx_v_11_gldraw_es2_cube.attrib_pointers[0]));

  /* "_gldraw_es2.pyx":282
 * 
 *     _gl_enable_pointer(ATTRIB_LOCATION,   3, GL_FLOAT, GL_FALSE, cube.attrib_pointers[0])
 *     _gl_enable_pointer(ATTRIB_LOCATION+1, 3, GL_FLOAT, GL_FALSE, cube.attrib_pointers[1])             # <<<<<<<<<<<<<<
 *     _gl_enable_pointer(ATTRIB_LOCATION+2, 3, GL_UNSIGNED_BYTE, GL_TRUE, cube.attrib_pointers[2])
 *     _gl_enable_pointer(ATTRIB_LOCATION+3, 2, GL_FLOAT, GL_FALSE, cube.attrib_pointers[3])
 */
  __pyx_f_11_gldraw_es2__gl_enable_pointer((__pyx_e_11_gldraw_es2_ATTRIB_LOCATION + 1), 3, GL_FLOAT, GL_FALSE, (__pyx_v_11_gldraw_es2_cube.attrib_pointers[1]));

  /* "_gldraw_es2.pyx":283
 *     _gl_enable_pointer(ATTRIB_LOCATION,   3, GL_FLOAT, GL_FALSE, cube.attrib_pointers[0])
 *     _gl_enable_pointer(ATTRIB_LOCATION+1, 3, GL_FLOAT, GL_FALSE, cube.attrib_pointers[1])
 *     _gl_enable_pointer(ATTRIB_LOCATION+2, 3, GL_UNSIGNED_BYTE, GL_TRUE, cube.attrib_pointers[2])             # <<<<<<<<<<<<<<
 *     _gl_enable_pointer(ATTRIB_LOCATION+3, 2, GL_FLOAT, GL_FALSE, cube.attrib_pointers[3])
 *     _gl_enable_pointer(ATTRIB_LOCATION+4, 3, GL_FLOAT, GL_FALSE, cube.attrib_pointers[4])
 */
  __pyx_f_11_gldraw_es2__gl_enable_pointer((__pyx_e_11_gldraw_es2_ATTRIB_LOCATION + 2), 3, GL_UNSIGNED_BYTE, GL_TRUE, (__pyx_v_11_gldraw_es2_cube.attrib_pointers[2]));

  /* "_gldraw_es2.pyx":284
 *     _gl_enable_pointer(ATTRIB_LOCATION+1, 3, GL_FLOAT, GL_FALSE, cube.attrib_pointers[1])
 *     _gl_enable_pointer(ATTRIB_LOCATION+2, 3, GL_UNSIGNED_BYTE, GL_TRUE, cube.attrib_pointers[2])
 *     _gl_enable_pointer(ATTRIB_LOCATION+3, 2, GL_FLOAT, GL_FALSE, cube.attrib_pointers[3])             # <<<<<<<<<<<<<<
 *     _gl_enable_pointer(ATTRIB_LOCATION+4, 3, GL_FLOAT, GL_FALSE, cube.attrib_pointers[4])
 *     _gl_enable_pointer(PICKATTRIB_LOCATION, 3, GL_FLOAT, GL_FALSE, cube.attrib_pointers[5])
 */
  __pyx_f_11_gldraw_es2__gl_enable_pointer((__pyx_e_11_gldraw_es2_ATTRIB_LOCATION + 3), 2, GL_FLOAT, GL_FALSE, (__pyx_v_11_gldraw_es2_cube.attrib_pointers[3]));

  /* "_gldraw_es2.pyx":285
 *     _gl_enable_pointer(ATTRIB_LOCATION+2, 3, GL_UNSIGNED_BYTE, GL_TRUE, cube.attrib_pointers[2])
 *     _gl_enable_pointer(ATTRIB_LOCATION+3, 2, GL_FLOAT, GL_FALSE, cube.attrib_pointers[3])
 *     _gl_enable_pointer(ATTRIB_LOCATION+4, 3, GL_FLOAT, GL_FALSE, cube.attrib_pointers[4])             # <<<<<<<<<<<<<<
 *     _gl_enable_pointer(PICKATTRIB_LOCATION, 3, GL_FLOAT, GL_FALSE, cube.attrib_pointers[5])
 *     _gl_enable_pointer(PICKATTRIB_LOCATION+1, 3, GL_UNSIGNED_BYTE, GL_TRUE, cube.attrib_pointers[6])
 */
  __pyx_f_11_gldraw_es2__gl_enable_pointer((__pyx_e_11_gldraw_es2_ATTRIB_LOCATION + 4), 3, GL_FLOAT, GL_FALSE, (__pyx_v_11_gldraw_es2_cube.attrib_pointers[4]));

  /* "_gldraw_es2.pyx":286
 *     _gl_enable_pointer(ATTRIB_LOCATION+3, 2, GL_FLOAT, GL_FALSE, cube.attrib_pointers[3])
 *     _gl_enable_pointer(ATTRIB_LOCATION+4, 3, GL_FLOAT, GL_FALSE, cube.attrib_pointers[4])
 *     _gl_enable_pointer(PICKATTRIB_LOCATION, 3, GL_FLOAT, GL_FALSE, cube.attrib_pointers[5])             # <<<<<<<<<<<<<<
 *     _gl_enable_pointer(PICKATTRIB_LOCATION+1, 3, GL_UNSIGNED_BYTE, GL_TRUE, cube.attrib_pointers[6])
 * 
 */
  __pyx_f_11_gldraw_es2__gl_enable_pointer(__pyx_e_11_gldraw_es2_PICKATTRIB_LOCATION, 3, GL_FLOAT, GL_FALSE, (__pyx_v_11_gldraw_es2_cube.attrib_pointers[5]));

  /* "_gldraw_es2.pyx":287
 *     _gl_enable_pointer(ATTRIB_LOCATION+4, 3, GL_FLOAT, GL_FALSE, cube.attrib_pointers[4])
 *     _gl_enable_pointer(PICKATTRIB_LOCATION, 3, GL_FLOAT, GL_FALSE, cube.attrib_pointers[5])
 *     _gl_enable_pointer(PICKATTRIB_LOCATION+1, 3, GL_UNSIGNED_BYTE, GL_TRUE, cube.attrib_pointers[6])             # <<<<<<<<<<<<<<
 * 
 * ##pxm-FUNC PD nogil
 */
  __pyx_f_11_gldraw_es2__gl_enable_pointer((__pyx_e_11_gldraw_es2_PICKATTRIB_LOCATION + 1), 3, GL_UNSIGNED_BYTE, GL_TRUE, (__pyx_v_11_gldraw_es2_cube.attrib_pointers[6]));

  /* "_gldraw_es2.pyx":277
 * 
 * ##pxm-FUNC PD nogil
 * cdef void gl_enable_data() nogil:             # <<<<<<<<<<<<<<
 *     glBindBuffer(GL_ARRAY_BUFFER, cube.glbuffer)
 *     glBufferData(GL_ARRAY_BUFFER, cube.vertexdatalen, cube.vertexdata, GL_STATIC_DRAW)
 */

  /* function exit code */
}

/* "_gldraw_es2.pyx":290
 * 
 * ##pxm-FUNC PD nogil
 * cdef void gl_disable_data() nogil:             # <<<<<<<<<<<<<<
 *     glBindBuffer(GL_ARRAY_BUFFER, 0)
 *     _gl_disable_pointer(ATTRIB_LOCATION)
 */

static void __pyx_f_11_gldraw_es2_gl_disable_data(void) {

  /* "_gldraw_es2.pyx":291
 * ##pxm-FUNC PD nogil
 * cdef void gl_disable_data() nogil:
 *     glBindBuffer(GL_ARRAY_BUFFER, 0)             # <<<<<<<<<<<<<<
 *     _gl_disable_pointer(ATTRIB_LOCATION)
 *     _gl_disable_pointer(ATTRIB_LOCATION+1)
 */
  glBindBuffer(GL_ARRAY_BUFFER, 0);

  /* "_gldraw_es2.pyx":292
 * cdef void gl_disable_data() nogil:
 *     glBindBuffer(GL_ARRAY_BUFFER, 0)
 *     _gl_disable_pointer(ATTRIB_LOCATION)             # <<<<<<<<<<<<<<
 *     _gl_disable_pointer(ATTRIB_LOCATION+1)
 *     _gl_disable_pointer(ATTRIB_LOCATION+2)
 */
  __pyx_f_11_gldraw_es2__gl_disable_pointer(__pyx_e_11_gldraw_es2_ATTRIB_LOCATION);

  /* "_gldraw_es2.pyx":293
 *     glBindBuffer(GL_ARRAY_BUFFER, 0)
 *     _gl_disable_pointer(ATTRIB_LOCATION)
 *     _gl_disable_pointer(ATTRIB_LOCATION+1)             # <<<<<<<<<<<<<<
 *     _gl_disable_pointer(ATTRIB_LOCATION+2)
 *     _gl_disable_pointer(ATTRIB_LOCATION+3)
 */
  __pyx_f_11_gldraw_es2__gl_disable_pointer((__pyx_e_11_gldraw_es2_ATTRIB_LOCATION + 1));

  /* "_gldraw_es2.pyx":294
 *     _gl_disable_pointer(ATTRIB_LOCATION)
 *     _gl_disable_pointer(ATTRIB_LOCATION+1)
 *     _gl_disable_pointer(ATTRIB_LOCATION+2)             # <<<<<<<<<<<<<<
 *     _gl_disable_pointer(ATTRIB_LOCATION+3)
 *     _gl_disable_pointer(ATTRIB_LOCATION+4)
 */
  __pyx_f_11_gldraw_es2__gl_disable_pointer((__pyx_e_11_gldraw_es2_ATTRIB_LOCATION + 2));

  /* "_gldraw_es2.pyx":295
 *     _gl_disable_pointer(ATTRIB_LOCATION+1)
 *     _gl_disable_pointer(ATTRIB_LOCATION+2)
 *     _gl_disable_pointer(ATTRIB_LOCATION+3)             # <<<<<<<<<<<<<<
 *     _gl_disable_pointer(ATTRIB_LOCATION+4)
 *     _gl_disable_pointer(PICKATTRIB_LOCATION)
 */
  __pyx_f_11_gldraw_es2__gl_disable_pointer((__pyx_e_11_gldraw_es2_ATTRIB_LOCATION + 3));

  /* "_gldraw_es2.pyx":296
 *     _gl_disable_pointer(ATTRIB_LOCATION+2)
 *     _gl_disable_pointer(ATTRIB_LOCATION+3)
 *     _gl_disable_pointer(ATTRIB_LOCATION+4)             # <<<<<<<<<<<<<<
 *     _gl_disable_pointer(PICKATTRIB_LOCATION)
 *     _gl_disable_pointer(PICKATTRIB_LOCATION+1)
 */
  __pyx_f_11_gldraw_es2__gl_disable_pointer((__pyx_e_11_gldraw_es2_ATTRIB_LOCATION + 4));

  /* "_gldraw_es2.pyx":297
 *     _gl_disable_pointer(ATTRIB_LOCATION+3)
 *     _gl_disable_pointer(ATTRIB_LOCATION+4)
 *     _gl_disable_pointer(PICKATTRIB_LOCATION)             # <<<<<<<<<<<<<<
 *     _gl_disable_pointer(PICKATTRIB_LOCATION+1)
 * 
 */
  __pyx_f_11_gldraw_es2__gl_disable_pointer(__pyx_e_11_gldraw_es2_PICKATTRIB_LOCATION);

  /* "_gldraw_es2.pyx":298
 *     _gl_disable_pointer(ATTRIB_LOCATION+4)
 *     _gl_disable_pointer(PICKATTRIB_LOCATION)
 *     _gl_disable_pointer(PICKATTRIB_LOCATION+1)             # <<<<<<<<<<<<<<
 * 
 * ##pxm-FUNC PD nogil
 */
  __pyx_f_11_gldraw_es2__gl_disable_pointer((__pyx_e_11_gldraw_es2_PICKATTRIB_LOCATION + 1));

  /* "_gldraw_es2.pyx":290
 * 
 * ##pxm-FUNC PD nogil
 * cdef void gl_disable_data() nogil:             # <<<<<<<<<<<<<<
 *     glBindBuffer(GL_ARRAY_BUFFER, 0)
 *     _gl_disable_pointer(ATTRIB_LOCATION)
 */

  /* function exit code */
}

/* "_gldraw_es2.pyx":301
 * 
 * ##pxm-FUNC PD nogil
 * cdef void gl_draw_cube_debug() nogil:             # <<<<<<<<<<<<<<
 *     cdef mat4 object_matrix #px/
 *     #object_matrix = mat4()
 */

static void __pyx_f_11_gldraw_es2_gl_draw_cube_debug(void) {
  __pyx_t_11_gldraw_es2_mat4 __pyx_v_object_matrix;

  /* "_gldraw_es2.pyx":304
 *     cdef mat4 object_matrix #px/
 *     #object_matrix = mat4()
 *     matrix_set_identity(object_matrix)             # <<<<<<<<<<<<<<
 *     glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, &object_matrix[0][0]) #px/
 *     #glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, object_matrix)
 */
  __pyx_f_11_gldraw_es2_matrix_set_identity(__pyx_v_object_matrix);

  /* "_gldraw_es2.pyx":305
 *     #object_matrix = mat4()
 *     matrix_set_identity(object_matrix)
 *     glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, &object_matrix[0][0]) #px/             # <<<<<<<<<<<<<<
 *     #glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, object_matrix)
 *     glDrawArrays(GL_LINES, cube.idx_debug, cube.cnt_debug)
 */
  glUniformMatrix4fv(__pyx_v_11_gldraw_es2_cube.object_location, 1, GL_FALSE, (&((__pyx_v_object_matrix[0])[0])));

  /* "_gldraw_es2.pyx":307
 *     glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, &object_matrix[0][0]) #px/
 *     #glUniformMatrix4fv(cube.object_location, 1, GL_FALSE, object_matrix)
 *     glDrawArrays(GL_LINES, cube.idx_debug, cube.cnt_debug)             # <<<<<<<<<<<<<<
 * 
 * ##pxm-FUNC PD nogil
 */
  glDrawArrays(GL_LINES, __pyx_v_11_gldraw_es2_cube.idx_debug, __pyx_v_11_gldraw_es2_cube.cnt_debug);

  /* "_gldraw_es2.pyx":301
 * 
 * ##pxm-FUNC PD nogil
 * cdef void gl_draw_cube_debug() nogil:             # <<<<<<<<<<<<<<
 *     cdef mat4 object_matrix #px/
 *     #object_matrix = mat4()
 */

  /* function exit code */
}

/* "_gldraw_es2.pyx":310
 * 
 * ##pxm-FUNC PD nogil
 * cdef void gl_draw_select_debug(GLfloat *selectdata, GLsizeiptr size, GLuint prog_hud) nogil:             # <<<<<<<<<<<<<<
 *     cdef int i, j #px+
 *     cdef GLintptr offset #px+
 */

static void __pyx_f_11_gldraw_es2_gl_draw_select_debug(GLfloat *__pyx_v_selectdata, GLsizeiptr __pyx_v_size, GLuint __pyx_v_prog_hud) {
  GLintptr __pyx_v_offset;
  int __pyx_t_1;

  /* "_gldraw_es2.pyx":313
 *     cdef int i, j #px+
 *     cdef GLintptr offset #px+
 *     if cube.cnt_debug == 0:             # <<<<<<<<<<<<<<
 *         #XXX: The data used here has fixed size, but may not be initialized yet
 *         return
 */
  __pyx_t_1 = ((__pyx_v_11_gldraw_es2_cube.cnt_debug == 0) != 0);
  if (__pyx_t_1) {

    /* "_gldraw_es2.pyx":315
 *     if cube.cnt_debug == 0:
 *         #XXX: The data used here has fixed size, but may not be initialized yet
 *         return             # <<<<<<<<<<<<<<
 *     offset = (cube.idx_debug+cube.cnt_debug) * 3 * sizeof(GLfloat)
 *     glBufferSubData(GL_ARRAY_BUFFER, offset, size, &selectdata[0]) #px/
 */
    goto __pyx_L0;

    /* "_gldraw_es2.pyx":313
 *     cdef int i, j #px+
 *     cdef GLintptr offset #px+
 *     if cube.cnt_debug == 0:             # <<<<<<<<<<<<<<
 *         #XXX: The data used here has fixed size, but may not be initialized yet
 *         return
 */
  }

  /* "_gldraw_es2.pyx":316
 *         #XXX: The data used here has fixed size, but may not be initialized yet
 *         return
 *     offset = (cube.idx_debug+cube.cnt_debug) * 3 * sizeof(GLfloat)             # <<<<<<<<<<<<<<
 *     glBufferSubData(GL_ARRAY_BUFFER, offset, size, &selectdata[0]) #px/
 *     #glBufferSubData(GL_ARRAY_BUFFER, offset, len(selectdata) * sizeof(GLfloat), ArrayDatatype.asArray(selectdata, GL_FLOAT))
 */
  __pyx_v_offset = (((__pyx_v_11_gldraw_es2_cube.idx_debug + __pyx_v_11_gldraw_es2_cube.cnt_debug) * 3) * (sizeof(GLfloat)));

  /* "_gldraw_es2.pyx":317
 *         return
 *     offset = (cube.idx_debug+cube.cnt_debug) * 3 * sizeof(GLfloat)
 *     glBufferSubData(GL_ARRAY_BUFFER, offset, size, &selectdata[0]) #px/             # <<<<<<<<<<<<<<
 *     #glBufferSubData(GL_ARRAY_BUFFER, offset, len(selectdata) * sizeof(GLfloat), ArrayDatatype.asArray(selectdata, GL_FLOAT))
 * 
 */
  glBufferSubData(GL_ARRAY_BUFFER, __pyx_v_offset, __pyx_v_size, (&(__pyx_v_selectdata[0])));

  /* "_gldraw_es2.pyx":320
 *     #glBufferSubData(GL_ARRAY_BUFFER, offset, len(selectdata) * sizeof(GLfloat), ArrayDatatype.asArray(selectdata, GL_FLOAT))
 * 
 *     glDisable(GL_DEPTH_TEST)             # <<<<<<<<<<<<<<
 *     glDrawArrays(GL_LINES, cube.idx_debug+cube.cnt_debug, 2)
 *     glUseProgram(prog_hud)
 */
  glDisable(GL_DEPTH_TEST);

  /* "_gldraw_es2.pyx":321
 * 
 *     glDisable(GL_DEPTH_TEST)
 *     glDrawArrays(GL_LINES, cube.idx_debug+cube.cnt_debug, 2)             # <<<<<<<<<<<<<<
 *     glUseProgram(prog_hud)
 *     glDrawArrays(GL_POINTS, cube.idx_debug+cube.cnt_debug+2, 2)
 */
  glDrawArrays(GL_LINES, (__pyx_v_11_gldraw_es2_cube.idx_debug + __pyx_v_11_gldraw_es2_cube.cnt_debug), 2);

  /* "_gldraw_es2.pyx":322
 *     glDisable(GL_DEPTH_TEST)
 *     glDrawArrays(GL_LINES, cube.idx_debug+cube.cnt_debug, 2)
 *     glUseProgram(prog_hud)             # <<<<<<<<<<<<<<
 *     glDrawArrays(GL_POINTS, cube.idx_debug+cube.cnt_debug+2, 2)
 * 
 */
  glUseProgram(__pyx_v_prog_hud);

  /* "_gldraw_es2.pyx":323
 *     glDrawArrays(GL_LINES, cube.idx_debug+cube.cnt_debug, 2)
 *     glUseProgram(prog_hud)
 *     glDrawArrays(GL_POINTS, cube.idx_debug+cube.cnt_debug+2, 2)             # <<<<<<<<<<<<<<
 * 
 * ##pxm-FUNC PD nogil
 */
  glDrawArrays(GL_POINTS, ((__pyx_v_11_gldraw_es2_cube.idx_debug + __pyx_v_11_gldraw_es2_cube.cnt_debug) + 2), 2);

  /* "_gldraw_es2.pyx":310
 * 
 * ##pxm-FUNC PD nogil
 * cdef void gl_draw_select_debug(GLfloat *selectdata, GLsizeiptr size, GLuint prog_hud) nogil:             # <<<<<<<<<<<<<<
 *     cdef int i, j #px+
 *     cdef GLintptr offset #px+
 */

  /* function exit code */
  __pyx_L0:;
}

/* "_gldraw_es2.pyx":326
 * 
 * ##pxm-FUNC PD nogil
 * cdef void gl_init_object_location(GLuint location) nogil:             # <<<<<<<<<<<<<<
 *     cube.object_location = location
 * 
 */

static void __pyx_f_11_gldraw_es2_gl_init_object_location(GLuint __pyx_v_location) {

  /* "_gldraw_es2.pyx":327
 * ##pxm-FUNC PD nogil
 * cdef void gl_init_object_location(GLuint location) nogil:
 *     cube.object_location = location             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_11_gldraw_es2_cube.object_location = __pyx_v_location;

  /* "_gldraw_es2.pyx":326
 * 
 * ##pxm-FUNC PD nogil
 * cdef void gl_init_object_location(GLuint location) nogil:             # <<<<<<<<<<<<<<
 *     cube.object_location = location
 * 
 */

  /* function exit code */
}

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    "_gldraw_es2",
    0, /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_DEBUG_MSGEXT, __pyx_k_DEBUG_MSGEXT, sizeof(__pyx_k_DEBUG_MSGEXT), 0, 0, 1, 1},
  {&__pyx_kp_u_GL_type_es2, __pyx_k_GL_type_es2, sizeof(__pyx_k_GL_type_es2), 0, 1, 0, 0},
  {&__pyx_kp_u_compiled, __pyx_k_compiled, sizeof(__pyx_k_compiled), 0, 1, 0, 0},
  {&__pyx_n_s_compiled_2, __pyx_k_compiled_2, sizeof(__pyx_k_compiled_2), 0, 0, 1, 1},
  {&__pyx_kp_u_from_package, __pyx_k_from_package, sizeof(__pyx_k_from_package), 0, 1, 0, 0},
  {&__pyx_n_s_gldraw_es2, __pyx_k_gldraw_es2, sizeof(__pyx_k_gldraw_es2), 0, 0, 1, 1},
  {&__pyx_kp_s_tmp, __pyx_k_tmp, sizeof(__pyx_k_tmp), 0, 0, 1, 0},
  {&__pyx_kp_u_init_module, __pyx_k_init_module, sizeof(__pyx_k_init_module), 0, 1, 0, 0},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_package, __pyx_k_package, sizeof(__pyx_k_package), 0, 0, 1, 1},
  {&__pyx_n_s_print, __pyx_k_print, sizeof(__pyx_k_print), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_set_debug_flags, __pyx_k_set_debug_flags, sizeof(__pyx_k_set_debug_flags), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_print = __Pyx_GetBuiltinName(__pyx_n_s_print); if (!__pyx_builtin_print) __PYX_ERR(0, 139, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 153, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "_gldraw_es2.pyx":142
 *         print('  from package:', __package__)
 *         print('  compiled:', __compiled)
 *         print('  GL-type: es2') #px+             # <<<<<<<<<<<<<<
 *     cube.number_blocks = 0
 *     cube.cnt_pick = 0
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_u_GL_type_es2); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "_gldraw_es2.pyx":60
 * debug = 0
 * 
 * def set_debug_flags(module):             # <<<<<<<<<<<<<<
 *     global debug
 *     if module.DEBUG_MSGEXT: debug |= DEBUG_MSGEXT
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_n_s_module); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);
  __pyx_codeobj__3 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__2, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_tmp, __pyx_n_s_set_debug_flags, 60, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__3)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC init_gldraw_es2(void); /*proto*/
PyMODINIT_FUNC init_gldraw_es2(void)
#else
PyMODINIT_FUNC PyInit__gldraw_es2(void); /*proto*/
PyMODINIT_FUNC PyInit__gldraw_es2(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit__gldraw_es2(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("_gldraw_es2", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main__gldraw_es2) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "_gldraw_es2")) {
      if (unlikely(PyDict_SetItemString(modules, "_gldraw_es2", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  if (__Pyx_ExportFunction("matrix_set_identity", (void (*)(void))__pyx_f_11_gldraw_es2_matrix_set_identity, "void (__pyx_t_11_gldraw_es2_vec4 *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("init_gldraw", (void (*)(void))__pyx_f_11_gldraw_es2_init_gldraw, "void (void)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("sync_block_transformations", (void (*)(void))__pyx_f_11_gldraw_es2_sync_block_transformations, "void (short *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("sync_animation_start", (void (*)(void))__pyx_f_11_gldraw_es2_sync_animation_start, "void (int, short *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("sync_animation_next", (void (*)(void))__pyx_f_11_gldraw_es2_sync_animation_next, "void (float, float, float, float)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("gl_draw_cube", (void (*)(void))__pyx_f_11_gldraw_es2_gl_draw_cube, "void (void)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("gl_pick_cube", (void (*)(void))__pyx_f_11_gldraw_es2_gl_pick_cube, "void (void)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("gl_init_buffers", (void (*)(void))__pyx_f_11_gldraw_es2_gl_init_buffers, "void (void)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("gl_delete_buffers", (void (*)(void))__pyx_f_11_gldraw_es2_gl_delete_buffers, "void (void)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("sync_blocks", (void (*)(void))__pyx_f_11_gldraw_es2_sync_blocks, "void (int, short *, int, int, int)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("sync_vertexdata", (void (*)(void))__pyx_f_11_gldraw_es2_sync_vertexdata, "void (int, char *, long *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("sync_transformations", (void (*)(void))__pyx_f_11_gldraw_es2_sync_transformations, "void (int, __pyx_t_11_gldraw_es2_mat4 *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("gl_enable_data", (void (*)(void))__pyx_f_11_gldraw_es2_gl_enable_data, "void (void)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("gl_disable_data", (void (*)(void))__pyx_f_11_gldraw_es2_gl_disable_data, "void (void)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("gl_draw_cube_debug", (void (*)(void))__pyx_f_11_gldraw_es2_gl_draw_cube_debug, "void (void)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("gl_draw_select_debug", (void (*)(void))__pyx_f_11_gldraw_es2_gl_draw_select_debug, "void (GLfloat *, GLsizeiptr, GLuint)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("gl_init_object_location", (void (*)(void))__pyx_f_11_gldraw_es2_gl_init_object_location, "void (GLuint)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Type init code ---*/
  /*--- Type import code ---*/
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "_gldraw_es2.pyx":25
 * # This line makes cython happy
 * global __name__, __package__    # pylint: disable=W0604
 * __compiled = True #px/             # <<<<<<<<<<<<<<
 * #__compiled = False
 * 
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_compiled_2, Py_True) < 0) __PYX_ERR(0, 25, __pyx_L1_error)

  /* "_gldraw_es2.pyx":58
 *     DEBUG_MSGEXT = 8
 * cdef long debug #px+
 * debug = 0             # <<<<<<<<<<<<<<
 * 
 * def set_debug_flags(module):
 */
  __pyx_v_11_gldraw_es2_debug = 0;

  /* "_gldraw_es2.pyx":60
 * debug = 0
 * 
 * def set_debug_flags(module):             # <<<<<<<<<<<<<<
 *     global debug
 *     if module.DEBUG_MSGEXT: debug |= DEBUG_MSGEXT
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_11_gldraw_es2_1set_debug_flags, NULL, __pyx_n_s_gldraw_es2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_set_debug_flags, __pyx_t_1) < 0) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "_gldraw_es2.pyx":1
 * #-*- coding:utf-8 -*-             # <<<<<<<<<<<<<<
 * # cython: profile=False
 * 
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init _gldraw_es2", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init _gldraw_es2");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* GetModuleGlobalName */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

/* PyObjectCall */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyErrFetchRestore */
  #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* WriteUnraisableException */
  static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* CodeObjectCache */
  static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
  #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    py_code = __pyx_find_code_object(c_line ? c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        __pyx_d,      /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_int(unsigned int value) {
    const unsigned int neg_one = (unsigned int) -1, const_zero = (unsigned int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(unsigned int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(unsigned int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(unsigned int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(unsigned int),
                                     little, !is_unsigned);
    }
}

/* CIntFromPyVerify */
  #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
  static CYTHON_INLINE unsigned int __Pyx_PyInt_As_unsigned_int(PyObject *x) {
    const unsigned int neg_one = (unsigned int) -1, const_zero = (unsigned int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(unsigned int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(unsigned int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (unsigned int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(unsigned int, digit, digits[0])
                case 2:
                    if (8 * sizeof(unsigned int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) >= 2 * PyLong_SHIFT) {
                            return (unsigned int) (((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) >= 3 * PyLong_SHIFT) {
                            return (unsigned int) (((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) >= 4 * PyLong_SHIFT) {
                            return (unsigned int) (((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (unsigned int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(unsigned int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(unsigned int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(unsigned int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(unsigned int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned int) (((unsigned int)-1)*(((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(unsigned int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned int) ((((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned int) (((unsigned int)-1)*(((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned int) ((((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned int) (((unsigned int)-1)*(((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned int) ((((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(unsigned int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            unsigned int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (unsigned int) -1;
        }
    } else {
        unsigned int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (unsigned int) -1;
        val = __Pyx_PyInt_As_unsigned_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to unsigned int");
    return (unsigned int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to unsigned int");
    return (unsigned int) -1;
}

/* CIntFromPy */
  static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
  static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CheckBinaryVersion */
  static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* FunctionExport */
  static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(__pyx_m, (char *)"__pyx_capi__");
    if (!d) {
        PyErr_Clear();
        d = PyDict_New();
        if (!d)
            goto bad;
        Py_INCREF(d);
        if (PyModule_AddObject(__pyx_m, (char *)"__pyx_capi__", d) < 0)
            goto bad;
    }
    tmp.fp = f;
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(tmp.p, sig, 0);
#else
    cobj = PyCObject_FromVoidPtrAndDesc(tmp.p, (void *)sig, 0);
#endif
    if (!cobj)
        goto bad;
    if (PyDict_SetItemString(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

/* InitStrings */
  static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if CYTHON_COMPILING_IN_CPYTHON && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
#if PY_VERSION_HEX < 0x03030000
        char* defenc_c;
        PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
        if (!defenc) return NULL;
        defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        {
            char* end = defenc_c + PyBytes_GET_SIZE(defenc);
            char* c;
            for (c = defenc_c; c < end; c++) {
                if ((unsigned char) (*c) >= 128) {
                    PyUnicode_AsASCIIString(o);
                    return NULL;
                }
            }
        }
#endif
        *length = PyBytes_GET_SIZE(defenc);
        return defenc_c;
#else
        if (__Pyx_PyUnicode_READY(o) == -1) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        if (PyUnicode_IS_ASCII(o)) {
            *length = PyUnicode_GET_LENGTH(o);
            return PyUnicode_AsUTF8(o);
        } else {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
#else
        return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
  #else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
  #endif
#else
  res = PyNumber_Int(x);
#endif
  if (res) {
#if PY_MAJOR_VERSION < 3
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%.4s__ returned non-%.4s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
